/******************************************************************************
 * HI 5323 Image Processing
 *
 * imageview.cpp
 ******************************************************************************/

/*
 * class name: ImageView
 * display the input image, and calculate the graylevel histogram
*/
#include <qapplication.h>
#include <qimage.h>
#include <qpainter.h>
#include <qcolor.h>
#include "histoview.h"

/**
 * class ImageView
 */
class ImageView : public QWidget
{

private:

    const QImage m_oImg;
    // array to hold the histogram pixel value
    ulong hgram[256];

public:

    ImageView( char *pFile, QWidget *pParent, const char *pName )
	: QWidget( pParent, pName ),
        m_oImg( pFile )
    {
    };

    /**
     * function to calculate the graylevel histogram
     */
    ulong* histogram()
    {
	int iX, iY, iMaxX, iMaxY;

	// initialize histogram array
	for (int i = 0; i < 256; i++) 
	    hgram[i] = 0;

	iX = 0;
	iY = 0;
        // obtain the image's width
	iMaxX = m_oImg.width();
        // obtain the image's height
	iMaxY = m_oImg.height();

	// variable to hold the graylevel value
	int iGraylevel;

	while (iY < iMaxY)
	{
	    while (iX < iMaxX)
	    {
                // return a pointer to the pixel data at "y" scan line
		uint *p = (uint *)m_oImg.scanLine(iY) + iX;
                // calculate graylevel for each pixel
		iGraylevel =(int)(0.299 * qRed(*p) + 0.587 * qGreen(*p)  + 0.114 * qBlue(*p) + 0.5);
		hgram[iGraylevel]++;
		iX++;
	    }
	    iX = 0;
	    iY++;
	}
	return hgram;
    }

    /**
     * display the original image
     */
    void paintEvent(QPaintEvent *)
    {
	QPainter p(this);
	p.drawImage(0,0,m_oImg);
    }

    /**
     * close the image by click mouse on image
     */
    void mousePressEvent ( QMouseEvent * )
    {
	close();
    }
};

int main( int argc, char **argv )
{
    QApplication app(argc, argv);

    // read original image as input
    ImageView oImgView("birds.jpg", 0, "birds");
    // where to draw the original image
    oImgView.setGeometry(280, 100, 740, 500);

    // initilize a HistoView object
    HistoView oHstView(oImgView.histogram());
    // where to draw the histogram
    oHstView.setGeometry(10, 10, 256, 256);

    app.setMainWidget(&oImgView);

    oImgView.show();
    oHstView.show();

    return app.exec();
}
